package sample.parser;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.lightysoft.logmx.business.ParsedEntry;
import com.lightysoft.logmx.mgr.LogFileParser;

/**
 * Sample LogMX Parser able to parse a log file with NO multi-line support and Relative Date support.<BR/>
 * The strings  #012  and  #015  are both replaced with \n (new line character)<BR/>
 * Here is an example of log file suitable for this parser:<BR/>
 * 
 *   May 02 10:01:02 MyThread INFO Emitter - This is a log message - Username : Company
 *   May 02 10:01:02 MyThread INFO Emitter - This is a log message...#012...on two lines - Username : Company
 *   May 02 10:01:02 MyThread INFO Emitter - This is a log message...#012...on three...#015...lines - Username : Company
 *   
 */
public class CustomSyslogParser extends LogFileParser {
    /** Current parsed log entry */
    private ParsedEntry entry = null;

    /** Entry date format */
    private SimpleDateFormat dateFormat = null;

    /** Entry date format string */
    private final static String DATE_FORMAT_STRING = "MMM dd HH:mm:ss";

    /** Mutex to avoid that multiple threads use the same Date formatter at the same time */
    private final Object DATE_FORMATTER_MUTEX = new Object();

    /** Pattern for entry begin */
    private final static Pattern ENTRY_BEGIN_PATTERN = Pattern.compile(
            "^(\\S+? \\d{2} \\d{2}:\\d{2}:\\d{2}) (\\S+?) (\\S+?) (\\S+?) - (.*) - (.*?) : (.*?)$");

    /** Pattern for tokens #012  and  #015  */
    private final static Pattern CRLF_PROTECTS_PATTERN = Pattern.compile("#012|#015");

    /** Key of user-defined field "Username" */
    private static final String EXTRA_USERNAME_FIELD_KEY = "Username";

    /** Key of user-defined field "Company" */
    private static final String EXTRA_COMPANY_FIELD_KEY = "Company";

    /** User-defined fields names (here, two) */
    private static final List<String> EXTRA_FIELDS_KEYS = Arrays.asList(EXTRA_USERNAME_FIELD_KEY,
            EXTRA_COMPANY_FIELD_KEY);


    /** 
     * Returns the name of this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getParserName()
     */
    @Override
    public String getParserName() {
        return "Custom Syslog Parser";
    }

    /**
     * Returns the supported file type for this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getSupportedFileType()
     */
    @Override
    public String getSupportedFileType() {
        return "Custom Syslog files";
    }

    /**
     * Process the new line of text read from file 
     * @see com.lightysoft.logmx.mgr.LogFileParser#parseLine(java.lang.String)
     */
    @Override
    protected void parseLine(String line) throws Exception {
        // If end of file, records last entry if necessary, and exits
        if (line == null) {
            recordPreviousEntryIfExists();
            return;
        }

        Matcher matcher = ENTRY_BEGIN_PATTERN.matcher(line);
        if (matcher.matches()) {
            // Record previous found entry if exists, then create a new one
            prepareNewEntry();

            entry.setDate(matcher.group(1));
            entry.setThread(matcher.group(2));
            entry.setLevel(matcher.group(3));
            entry.setEmitter(matcher.group(4));
            entry.setMessage(CRLF_PROTECTS_PATTERN.matcher(matcher.group(5)).replaceAll("\n"));
            entry.getUserDefinedFields().put(EXTRA_USERNAME_FIELD_KEY, matcher.group(6));
            entry.getUserDefinedFields().put(EXTRA_COMPANY_FIELD_KEY, matcher.group(7));
        }
    }

    /** 
     * Returns the ordered list of user-defined fields to display (given by their key), for each entry.
     * @see com.lightysoft.logmx.mgr.LogFileParser#getUserDefinedFields()
     */
    @Override
    public List<String> getUserDefinedFields() {
        return EXTRA_FIELDS_KEYS;
    }

    /**
     * Returns a relative Date for the given entry (implementation actually uses the Absolute date, which is technically relative to the year 0)
     * @see com.lightysoft.logmx.mgr.LogFileParser#getRelativeEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getRelativeEntryDate(ParsedEntry pEntry) throws Exception {
        return getAbsoluteEntryDate(pEntry);
    }

    /**
     * Returns the absolute Date for the given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getAbsoluteEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getAbsoluteEntryDate(ParsedEntry pEntry) throws Exception {
        if (dateFormat == null) {
            // Now create the date formatter using the right Locale
            // (method "getLocale()" can't be called from the constructor) 
            dateFormat = new SimpleDateFormat(DATE_FORMAT_STRING, getLocale());
        }
        synchronized (DATE_FORMATTER_MUTEX) { // Java date formatter is not thread-safe
            return dateFormat.parse(pEntry.getDate());
        }
    }

    /**
     * Send to LogMX the current parsed log entry
     * @throws Exception
     */
    private void recordPreviousEntryIfExists() throws Exception {
        if (entry != null) {
            addEntry(entry);
        }
    }

    /**
     * Send to LogMX the current parsed log entry, then create a new one
     * @throws Exception
     */
    private void prepareNewEntry() throws Exception {
        recordPreviousEntryIfExists();
        entry = createNewEntry();
        entry.setUserDefinedFields(new HashMap<String, Object>(2)); // Create an empty Map with 2 allocated entries
    }
}
